
  '==============
  'WAVEFORM AUDIO
  '==============

  /*
  Wave audio
  https://ccrma.stanford.edu/courses/422/projects/WaveFormat/
  '
  http://msdn.microsoft.com/en-us/library/ms713619.aspx
  functions
  http://msdn.microsoft.com/en-us/library/ms713504.aspx
  example
  http://msdn.microsoft.com/en-us/library/ms708482.aspx

  #case capital

  'FROM MMSYSTEM.H

  /* types for wType field in MMTIME struct */
  #define TIME_MS         0x0001  /* time in milliseconds */
  #define TIME_SAMPLES    0x0002  /* number of wave samples */
  #define TIME_BYTES      0x0004  /* current byte offset */
  #define TIME_SMPTE      0x0008  /* SMPTE time */
  #define TIME_MIDI       0x0010  /* MIDI time */
  #define TIME_TICKS      0x0020  /* Ticks within MIDI stream */


  /* defines for dwFormat field of WAVEINCAPS and WAVEOUTCAPS */
  #define WAVE_INVALIDFORMAT     0x00000000       /* invalid format */
  #define WAVE_FORMAT_1M08       0x00000001       /* 11.025 kHz, Mono,   8-bit  */
  #define WAVE_FORMAT_1S08       0x00000002       /* 11.025 kHz, Stereo, 8-bit  */
  #define WAVE_FORMAT_1M16       0x00000004       /* 11.025 kHz, Mono,   16-bit */
  #define WAVE_FORMAT_1S16       0x00000008       /* 11.025 kHz, Stereo, 16-bit */
  #define WAVE_FORMAT_2M08       0x00000010       /* 22.05  kHz, Mono,   8-bit  */
  #define WAVE_FORMAT_2S08       0x00000020       /* 22.05  kHz, Stereo, 8-bit  */
  #define WAVE_FORMAT_2M16       0x00000040       /* 22.05  kHz, Mono,   16-bit */
  #define WAVE_FORMAT_2S16       0x00000080       /* 22.05  kHz, Stereo, 16-bit */
  #define WAVE_FORMAT_4M08       0x00000100       /* 44.1   kHz, Mono,   8-bit  */
  #define WAVE_FORMAT_4S08       0x00000200       /* 44.1   kHz, Stereo, 8-bit  */
  #define WAVE_FORMAT_4M16       0x00000400       /* 44.1   kHz, Mono,   16-bit */
  #define WAVE_FORMAT_4S16       0x00000800       /* 44.1   kHz, Stereo, 16-bit */

  #define WAVE_FORMAT_44M08      0x00000100       /* 44.1   kHz, Mono,   8-bit  */
  #define WAVE_FORMAT_44S08      0x00000200       /* 44.1   kHz, Stereo, 8-bit  */
  #define WAVE_FORMAT_44M16      0x00000400       /* 44.1   kHz, Mono,   16-bit */
  #define WAVE_FORMAT_44S16      0x00000800       /* 44.1   kHz, Stereo, 16-bit */
  #define WAVE_FORMAT_48M08      0x00001000       /* 48     kHz, Mono,   8-bit  */
  #define WAVE_FORMAT_48S08      0x00002000       /* 48     kHz, Stereo, 8-bit  */
  #define WAVE_FORMAT_48M16      0x00004000       /* 48     kHz, Mono,   16-bit */
  #define WAVE_FORMAT_48S16      0x00008000       /* 48     kHz, Stereo, 16-bit */
  #define WAVE_FORMAT_96M08      0x00010000       /* 96     kHz, Mono,   8-bit  */
  #define WAVE_FORMAT_96S08      0x00020000       /* 96     kHz, Stereo, 8-bit  */
  #define WAVE_FORMAT_96M16      0x00040000       /* 96     kHz, Mono,   16-bit */
  #define WAVE_FORMAT_96S16      0x00080000       /* 96     kHz, Stereo, 16-bit */

  /* flags for wFormatTag field of WAVEFORMAT */

  #define WAVE_FORMAT_PCM     1


  typedef word  WORD
  typedef dword DWORD,*DWORD_PTR,MMVERSION
  typedef zstring TCHAR,*LPSTR


  typedef struct { 
    WORD  wFormatTag; 
    WORD  nChannels; 
    DWORD nSamplesPerSec; 
    DWORD nAvgBytesPerSec; 
    WORD  nBlockAlign; 
    WORD  wBitsPerSample; 
    WORD  cbSize; 
  } WAVEFORMATEX;


  typedef struct wavehdr_tag { 
    LPSTR      lpData; 
    DWORD      dwBufferLength; 
    DWORD      dwBytesRecorded; 
    DWORD_PTR  dwUser; 
    DWORD      dwFlags; 
    DWORD      dwLoops; 
    struct wavehdr_tag * lpNext; 
    DWORD_PTR reserved; 
  } WAVEHDR, *LPWAVEHDR; 


  typedef struct mmtime_tag { 
    UINT wType; 
    union { 
        DWORD ms; 
        DWORD sample; 
        DWORD cb; 
        DWORD ticks; 
        struct { 
            BYTE hour; 
            BYTE min; 
            BYTE sec; 
            BYTE frame; 
            BYTE fps; 
            BYTE dummy; 
            BYTE pad[2] 
        } smpte; 
        struct { 
            DWORD songptrpos; 
        } midi; 
    } u; 
  } MMTIME;


  typedef struct { 
    WORD      wMid; 
    WORD      wPid; 
    MMVERSION vDriverVersion; 
    TCHAR     szPname[MAXPNAMELEN]; 
    DWORD     dwFormats; 
    WORD      wChannels; 
    WORD      wReserved1; 
  } WAVEINCAPS; 


  typedef struct { 
    WORD      wMid; 
    WORD      wPid; 
    MMVERSION vDriverVersion; 
    TCHAR     szPname[MAXPNAMELEN]; 
    DWORD     dwFormats; 
    WORD      wChannels; 
    WORD      wReserved1; 
    DWORD     dwSupport; 
  } WAVEOUTCAPS; 


  WINMMAPI UINT WINAPI waveOutGetNumDevs(void);
  WINMMAPI MMRESULT WINAPI waveOutGetDevCapsA( __in UINT_PTR uDeviceID, __out LPWAVEOUTCAPSA pwoc, __in UINT cbwoc);
  WINMMAPI MMRESULT WINAPI waveOutGetVolume( __in_opt HWAVEOUT hwo, __out LPDWORD pdwVolume);
  WINMMAPI MMRESULT WINAPI waveOutSetVolume( __in_opt HWAVEOUT hwo, __in DWORD dwVolume);
  WINMMAPI MMRESULT WINAPI waveOutGetErrorTextA( __in MMRESULT mmrError, __out_ecount(cchText) LPSTR pszText, __in UINT cchText);
  WINMMAPI MMRESULT WINAPI waveOutGetErrorTextW( __in MMRESULT mmrError, __out_ecount(cchText) LPWSTR pszText, __in UINT cchText);
  WINMMAPI MMRESULT WINAPI waveOutOpen( __out_opt LPHWAVEOUT phwo, __in UINT uDeviceID,
    __in LPCWAVEFORMATEX pwfx, __in_opt DWORD_PTR dwCallback, __in_opt DWORD_PTR dwInstance, __in DWORD fdwOpen);
  WINMMAPI MMRESULT WINAPI waveOutClose( __in HWAVEOUT hwo);
  WINMMAPI MMRESULT WINAPI waveOutPrepareHeader( __in HWAVEOUT hwo, __inout_bcount(cbwh) LPWAVEHDR pwh,  __in UINT cbwh);
  WINMMAPI MMRESULT WINAPI waveOutUnprepareHeader( __in HWAVEOUT hwo, __inout_bcount(cbwh) LPWAVEHDR pwh, __in UINT cbwh);
  WINMMAPI MMRESULT WINAPI waveOutWrite( __in HWAVEOUT hwo, __inout_bcount(cbwh) LPWAVEHDR pwh, __in UINT cbwh);
  WINMMAPI MMRESULT WINAPI waveOutPause( __in HWAVEOUT hwo);
  WINMMAPI MMRESULT WINAPI waveOutRestart( __in HWAVEOUT hwo);
  WINMMAPI MMRESULT WINAPI waveOutReset( __in HWAVEOUT hwo);
  WINMMAPI MMRESULT WINAPI waveOutBreakLoop( __in HWAVEOUT hwo);
  WINMMAPI MMRESULT WINAPI waveOutGetPosition( __in HWAVEOUT hwo, __inout_bcount(cbmmt) LPMMTIME pmmt, __in UINT cbmmt);
  WINMMAPI MMRESULT WINAPI waveOutGetPitch( __in HWAVEOUT hwo, __out LPDWORD pdwPitch);
  WINMMAPI MMRESULT WINAPI waveOutSetPitch( __in HWAVEOUT hwo, __in DWORD dwPitch);
  WINMMAPI MMRESULT WINAPI waveOutGetPlaybackRate( __in HWAVEOUT hwo, __out LPDWORD pdwRate);
  WINMMAPI MMRESULT WINAPI waveOutSetPlaybackRate( __in HWAVEOUT hwo, __in DWORD dwRate);
  WINMMAPI MMRESULT WINAPI waveOutGetID( __in HWAVEOUT hwo, __out LPUINT puDeviceID);


