'
'
'
'         ###############
'       ###################
'      ###               ###
'     ###    ###          ###
'    ###    ## ##         ###
'    ###   ##   ##        ###
'    ###  ##     ##       ###
'    ###  ##      ##      ###
'    ###  ##      ##      ###
'    ###   ####  ##      ###
'     ###    ####       ###
'      ###             ###
'       ###           ###
'         #############
'           #########





'===========
kernel_funs:
'===========
'===========





'===========================
'>>O2 LOW LEVEL MACROS
'===========================


data       "#view 1 ###"
data       "#endv 1 ###"
'action:   creates a window for compiler listings.
'use:      Allows the programmer to see a selected part of the compiled code
'example:  dim a,b,c
'          ###
'          a=b+c
'          ###
'          s="Value "+str(a)
'remarks:  These markers restrict the listing to a window so that the code can be studied in detail.
'          #view and #endv are generated internally from the pair of '###' symbols.
'group:    directives



data       "version 1 0xA038"
'action:   returns hexadecimal coding of current version number.
'use:      identification of version
'example:  print hex version
'remarks:  This is updated whever a new version is released, and corresponds with the entries
'          in the development log included in the main oxygen folder.
'related:
'group:    info


'======================
'>>O2H LOW LEVEL MACROS
'======================


data       "new 1 %1 * %2 : @%2=getmemory sizeof %1 : %2.constructor"
'action:   create a dynamic object and call its constructor method.
'use:      to create and initialise persistent objects
'example:  new shape cuboid 1,1,1
'related:  del
'group:    system macro



data       "del 1 %1.destructor : freememory @%1 : @%1=0"
'action:   Call a dynamic object's destructor method and disallocate its memorey block.
'use:      to delete persistent objects
'example:  del cuboid
'related:  new
'group:    system macro



'===============================
'>>O2H SYSTEM TERMINATION MACROS
'===============================



data       "terminate 19 push eax : call [ebx+2448] : call [ebx+2144] : pop eax"+cr
'action:   Deallocate all Strings, static variables and free DLL libraries (at end of program).
'          This is automatically appended unless it is detected in the program.
'          It is composed of the three macros below: FreeLibs FreeStrings. If either
'          of these are used then Terminate itself will not be automatically appended.
'use:      explicit termination of program
'example:  terminate
'related:  freestrings freelibs
'group:    system macro


data       "freestrings 19 push eax : call [ebx+2144] : pop eax"
'action:   deallocate all remaining static strings only (at end of program)
'use:      allow the program to terminate without freeing libraries
'example:  freestrings
'related:  terminate freelibs
'group:    system macro



data       "freelibs 19 push eax : call [ebx+2448] : pop eax"
'action:   free all loaded DLL libraries only (at end of program)
'use:      allow the program to terminate but all static strings are retained
'example:  freelibs
'related:  terminate freestrings
'group:    system macro




'========================
'>>O2 LOW LEVEL FUNCTIONS
'========================



data       "getmemory 4 8"
'action:   allocate block of memory and return its base address
'use:      address=getmemory bytes
'example:  m=getmemory 8000
'related:  freememory, news, frees
'group:    low level oxygen functions



data       "freememory 4 16"
'action:   free previously allocated memory block
'use:      freememory address
'example:  freememory m
'related:  getmemory, news, frees
'group:    low level oxygen functions



data       "loadlibrary 4 24"
'action:   load a library (if not already loaded) and return its handle
'use:      handle=loadlibrary "library filename"
'example:  h=loadlibrary "kernel32.dll"
'related:  freelibrary, library, getprocaddress
'group:    low level oxygen functions


data       "freelibrary 4 32"
'action:   free library
'use:      freelibrary handle
'example:  freelibrary h
'related:  loadlibrary, library, getprocaddress
'group:    low level oxygen functions



data       "getprocaddress 4 40"
'action:   get procedure address
'use:      getprocaddress libraryHandle "proc name"
'example:  a=getprocaddress h, "AllocConsole"
'related:  loadlibrary, freelibrary, library
'group:    low level oxygen functions



data       "copy0 4 56"
'action:   copy null terminated string to another location
'use:      copy DestinationAddress, SourceAddress
'example:  copy &a,&b
'related:  copy00 copy
'group:    low level oxygen functions



data       "copy00 4 64"
'action:   copy null terminated string of wide (2 byte) characters to another location
'use:      copy DestinationAddress, SourceAddress
'example:  copy &v,&w
'related:  copy0, copy, copyn
'group:    low level oxygen functions



data       "copyn 4 72"
data       "copy 4 72"
'action:   string to another location by a specified number of bytes
'use:      copy DestinationAddress, SourceAddress, NumberOfBytes
'example:  copy &dest,&src,n
'remarks:  copy and copyn are the same
'related:  copy0 copy00
'group:    low level oxygen functions



data       "mbox 4 2048" 'mboxa
'action:   display message null-terminated ascii string
'use:      mbox StringAddress
'example:  mbox "okay"
'related:  print
'group:    low level oxygen functions



data       "comparestr 4 2192" ' compsa ' COMPARING BSTRS
'action:   compare first string with second string
'use:      comparestr bstr1,bstr2 : jg greater : jl less : jz equal      
'example:  comparestr bs1,bs2
'related:  str, ( < = > )
'group:    low level oxygen functions


'=============================
'>>COMMON RESERVED WORDS
'=============================

data "db     9 1"
data "dw     9 1"
data "dd     9 1"
data "dq     9 1"
data "exit   9 1"
data "repeat 9 1"
data "mov    9 0"
data "lea    9 0"
data "add    9 0"
data "sub    9 1"
data "mul    9 0"
data "div    9 0"
data "jmp    9 1"
data "jz     9 0"
data "jnz    9 0"
data "call   9 1"
data "and    9 0"
data "or     9 0"
data "xor    9 0"
data "not    9 1"
data "shl    9 0"
data "shr    9 0"
data "sal    9 0"
data "sar    9 0"
'
'groupref: common_asm
'group:    common x86 assembly instructions


''neg,test,ja,jb,jg,jl,jae,jbe,jge,jle




'===========================
'>>O2 ATTRIBUTE MACROS
'===========================



data       "sizeof 11 1"
'action:   return length of variable element (in bytes)
'use:      nbytes=sizeof variable
'example:  n=sizeof a
'related:  offsetof, spanof, typeof typecodeof
'group:    variable and type attributes



data       "offsetof 11 2"
'action:   return offset of variable from index register
'use:      nbytes=offsetof variable
'example:  n=offsetof a
'related:  sizeof, spanof, typeof typecodeof recordof
'group:    variable and type attributes



data       "spanof 11 3"
'action:   return span of array variable dimension
'use:      nbytes=spanof variable dimension
'example:  dim as long v(10) : n=spanof v
'result:   n=10
'related:  sizeof, offsetof, typeof typecodeof recordof
'group:    variable and type attributes



data       "typeof 11 4"
'action:   return name of the variable type
'use:      name=typeof variable
'example:  dim as long v : s=typeof v
'result:   s="long"
'related:  typecodeof sizeof, offsetof, spanof, recordof
'group:    variable and type attributes



data       "structureof 11 5"
'action:   return data structure of compund (UDT) or variabe
'use:      obtain data for diagnostics or reflective programming
'example:  type vt long v,double d
'          dim as vt v :  r=recordof v
'result:   r="
'          v 0 4 1 A0 , long
'          d 4 4 1 A0 , double
'          "
'related:  prototypeof sizeof, offsetof, spanof, typeof typecodeof recordof #recordof
'group:    variable and type attributes



data       "encodingof  11 5"
'action:   return the operand encoding of a variable or macro
'use:      obtain data for diagnostics or reflective programming
'example:  sys v
'          print encodingof v 'example: [ebx+4100]
'related:  prototypeof sizeof, offsetof, spanof, typeof typecodeof recordof #recordof
'group:    variable and type attributes



data       "prototypeof  11 6"
'action:   return prototype(s) of functions, subs and high level macros
'use:      obtain data for diagnostics or reflective programming
'related:  recordof sizeof, offsetof, spanof, typeof typecoedof #recordof
'group:    variable and type attributes



data       "typecodeof  11 7"
'action:   return type code number of variables and literals.
'use:      obtain data for diagnostics or reflective programming
'related:  typeof recordof sizeof, offsetof, spanof, prototypeof #recordof
'group:    variable and type attributes



data       "recordof    11 8"
'action:   return record of compund (UDT) variable
'use:      obtain data for diagnostics or reflective programming
'example:  type vt long v,double d
'          dim as vt v :  r=recordof v
'result:   r="
'          v 0 4 1 A0 , long
'          d 4 4 1 A0 , double
'          "
'related:  #recordof prototypeof sizeof, offsetof, spanof, typeof typecodeof #recordof
'group:    variable and type attributes
'remarks:  use #recordof to display the record during comolation.



'===========================
'>>O2 KEYWORDS
'===========================




data       "incl 13 1"
'action:   expand macro non recursively without parameter substitutions
'use:      incl aMacro   
'example:  def helo "hello " %1
'          incl helo
'result:   substitution: "hello " %1
'related:  def
'group:    Oxygen keywords


data       "#semicolon 13 2"
'action:   switch use of semicolon as comment marker or separator
'use:      so semicolons can be recognised as separators when using C syntax
'example:  #semicolon separator
'          s="ok"  ; print s
'          #semicolon comment
'          print "ok" ; this is a comment
'remarks:  #semicolon can be confined to a scope (function etc)
'related:  #case indexbase
'group:    Oxygen keywords


data       "indexers 13 3"
'action:   specify register to be used to access variables
'use:      used internally for specifying register-based offsets in Basic      
'example:  indexers ebp offset -8 descending
'related:  offset
'remarks:  Normally used internally but available for explicit use in Assembly coding.
'group:    Oxygen keywords



data       "retn 13 4"
'action:   macro ret n. Releases correct number of bytes at end of procedure (in stdcall convention)
'use:      
'example:  
'group:    Oxygen keywords



data       "offset 13 5"
'action:   specifty base offset to be used to access variables
'use:      
'example:  
'related:  indexers
'group:    Oxygen keywords



data       "skip 13 6"
'action:   start a skip block.
'use:      prevent code within the block from being executed
'example:  
'related:  rem, ('), (/* .. */), (//)
'group:    control structures
'group:    Oxygen keywords



data       "rem 13 25"
'action:   comment till end of line
'use:      
'example:  rem this is a comment
'related:  skip, ('), (/* .. */), (//)
'group:    Oxygen keywords



data       "with 13 7"
'action:   specify a variable name prefix for assignments
'use:      Allows long names to be shortened when assigning values
'example:  dim as vector4 vectorI
'          with vectorI. : x=1 : y=0 : z=13 : w=0 : end with
'group:    Oxygen keywords



data       "dataspace 13 8"
'action:   create an inline space for storage
'use:      
'example:  p=dataspace 1024
'          copy0 p,"This could be used as a static string buffer"
'          mbox p
'related:  data
'group:    Oxygen keywords


data       "data 13 9"
'action:   put o2 scripted data into data section and return
'          base address in register EAX
'use:      tables or series of const data (including strings
'example:
'          data
'          (
'            ns 1.0 ns 0.8 ns 0.8 ns 1.0
'            ns 0.6 ns 0.6 ns 0.6 ns 1.0
'            ns 20.0
'          )
'          mov edx,eax
'related:  o2
'group:    Oxygen keywords




data       "o2 13 10"
'action:   start a block of o2 machine code notation
'use:      inserting inline machine code and other binary data.
'          direct use of o2 language (used by the o2 linker)
'example:  o2 b8 00 01 00 00 'machine code for mov eax,256
'related:  asm
'group:    Oxygen keywords


data       "bind 13 12"
'action:   bind a list of procedures from a Dynamic Link Library (DLL)
'use:      for low level (without protoype) calls to DLL functions.
'          the first name is the one used in the program. The second is the name used in the DLL.
'example:
'          sys Kernel32
'          kernel32=LoadLibrary "kernel32.dll"
'
'          bind kernel32
'          (
'          GetExitCodeProcess GetExitCodeProcess           ; @8
'          ExitProcess        ExitProcess                  ; @4
'          GetCommandLine     GetCommandLineA              ; @0
'          GetModuleHandle    GetModuleHandleA             ; @4
'          QueryPerformanceCounter QueryPerformanceCounter ; @4
'          )
'result:   the first column of keywords is recognised as procedure calls. A prototype may be attached
'          to any of these keywords later. Otherwise the programmer must ensure that the parameters
'          passed are a perfect match for each procedure call.
'remarks:  Comments are supported - both semicolon and single quote marks.
'related:  loadlibrary, freelibrary, getprocaddress, declare, library
'group:    Oxygen keywords
  


data       "defs 13 14"
'action:   create an enumeration
'use:      assign a numeric identity to a name (same as enum)
'example:  
'          defs bit thing
'          (
'          Shoes,Ships, SealingWax, Cabbages, Kings
'          )
'          thing t
'          t=cabbages
'          print t
'          if t=cabbages then print "Cabbages!"
'
'related:  enum, def, #define, (%), typedef enum
'group:    Oxygen keywords



data       "def 13 15"
data       "#def 13 15"
'action:   define a low level macro
'use:      
'example: 
'          'DEFINE MACRO:
'
'          def Create
'            dim as %1 %2
'            %2.new("%2")
'          end def
'
'          'INVOKE MACRO
'
'          create SomeClass SomeObject
'          
'          'EXPANDS TO:
'
'          dim as SomeClass SomeObject
'          SomeObject.new("SomeObject")
'
'remarks:  def and #def are the same
'related:  macro, #define, defs deff
'group:    Oxygen keywords



data       "type 13 16"
'action:   define a compound variable type
'use:      
'example:
'          type rgbacolor
'            red   as byte
'            green as byte
'            blue  as byte
'            alpha as byte
'          end type
'
'          rgbacolor c
'          c.red=100 : c.green=50 : c.blue=100
'
'related:  typedef, struct, class
'group:    Oxygen keywords
 


data       "struct 13 11"
'action:   define a compound variable type (C Syntax)
'use:      
'example:  
'          struct rgbacolor
'          {
'            red   as byte
'            green as byte
'            blue  as byte
'            alpha as byte
'          }
'related:  type, typedef, class
'group:    Oxygen keywords



data       "class 13, 13"
'action:   define a class (strucure and methods for objects)
'use:      
'example:
'          'DEFINE CLASS
'          '  
'          class rgbacolor
'            red   as byte
'            green as byte
'            blue  as byte
'            alpha as byte
'            '
'            method in(sys r, sys g, sys b, sys a)
'              red=r : green=g : blue=b : alpha=a
'            end method
'            '
'          end class
'
'          'INSTANTIATE
'
'          dim as rgbaColor c
'
'          'INVOKE METHOD
'          '
'          c.in(100,200,100,128)
'
'related:  type, typedef, struct
'group:    Oxygen keywords



data       "var 13 17"
'action:   define a set of variables
'use:      low-level dimensioning of variables
'example:  var string s,t,u(64),v  
'remarks:  var is normally only used internally. it accepts * for indirect variables
'          and "at" for variable mapping. Arrays are also supported.
'related:  dim
'group:    Oxygen keywords



data       "include 13 18"
data       "#include 13 18"
'action:   include source code from another file
'use:      used for including header files and other units of source code
'example:  #include "rtl32.inc"
'remarks:  include and #include are the same
'related:  includepath
'group:    Oxygen keywords




data       "asm 13 19"
data       "#asm 13 19"
'group:    directives
'action:   switch to assembly code mode (excludes BASIC keywords)
'use:      used when scripting a section of assembly code only.
'example:  asm
'remarks:  asm and #asm are the same.
'          
'          if the script also uses basic, it must end with then "basic" instruction
'          to generate the Basic intialisation and termination code blocks
'related:  basic, o2h
'group:    Oxygen keywords



data       "o2h 13 20"
data       "#o2h 13 20"
data       "basic 13 20"
data       "#basic 13 20"
'group:    directives
'action:   switch to BASIC mode
'use:      enable support for BASIC and other high level scripting
'example:  basic
'remarks:  o2h #o2h basic and #basic are the same.
'related:  asm
'group:    Oxygen keywords



data       "packed 13 21"
'action:   prevent padding between members of a type or typedef
'use:      allow programmer to have full control over padding to align the members
'example:  packed type point word x,y,z, dword color
'          packed type point word x,y,z, align 4 dword color
'remarks:  By default, the members will be aligned to a boundary matching their size.
'related:  type typedef struct
'group:    Oxygen keywords



data       "_def 13 22"
'action:   define a low level system macro
'remarks:  the terminate, freelibs and freestatics macros are specified in RTL64
'group:    Oxygen keywords



data       "includepath 13 30"
'action:   define a filepath for source files specified by Include.
'use:      
'example:  
'related:  include
'group:    Oxygen keywords



data       "librarypath 13 31"
'action:   define a filepath for DLL files
'use:      tells the computer where to find Dynamic Link Libraries
'          required to run the program. (System DLLs  do not require this.)
'example:  
'related:  library, extern, includepath
'group:    Oxygen keywords


data       "#file 13 35"
'action:   specifiy a filename for compiled code ( .EXE or .DLL )
'use:      creates an executable ".EXE" or  Dynamic Link Library ".DLL" file
'example:  #file "4PortViewer.exe"
'related:  #economy
'group:    Oxygen keywords



data       "quote 13 36"
'action:   specify tagname for superquotes
'use:      supports almost unlimited nesting of quotes. Can be used for containg
'          embedded scripts which themselves use quotes and have different syntax.
'example:  quote !!! this quote contains other quotes "containing "other quotes"   " !!!
'related:  ("), (`)
'group:    Oxygen keywords


data       "typedef 13 37"
'action:   define a set of types (C syntax)
'use:      create type definitions for creating other types
'example:
'          typedef Double *pDouble
'example:  
'
'          'CREATE TYPES FROM ANONYMOUS STRUCT
'
'          typedef struct
'           {
'             single x;
'             single y;
'             single z;
'           } vector, *pVector;
'
'           'CREATE VARIABLE
'
'           vector v;
'
'           v.x=42.01
'
'example:   'from winbase.h
'           typedef struct _OVERLAPPED {
'               ULONG_PTR Internal;
'               ULONG_PTR InternalHigh;
'               union {
'                   struct {
'                       DWORD Offset;
'                       DWORD OffsetHigh;
'                   };
'
'                   PVOID Pointer;
'               };
'
'               HANDLE  hEvent;
'           } OVERLAPPED, *LPOVERLAPPED;
'
'related:  type, struct, class, union, enum
'group:    Oxygen keywords





data       "union 13 38"
'action:   define a union (C syntax)
'use:      allows different variables to occupy the same space.
'example:  
'          union utype
'          {
'            byte  b
'            short w
'            long  i
'          }
'
'
'          utype v
'
'          v.b=42
'result:   v.b=42 : v.w=42 : v.i=42
'remarks:  a union may also be nested inside a type.
'related:  typedef, type, struct, class, enum
'group:    Oxygen keywords



data       "#case 13 41"
'action:   specify mode of case sensitivity.
'use:      case can be 'sensitive' insensitive' or 'capital'
example:   
'          #case insensitive 'AbC is the same as 'aBC' and 'ABC'
'          #case sensitive   'AbC and 'abc' are not the same
'          #case capital     'AbC is the same as 'aBC' but not 'ABC'
'remarks:  The default is 'insensitive'.
'          #case has block scope and reverts to its previous state when
'          the block ends.
'related:  #semicolon indexbase
'group:    directives



data       "leaf 13 42"
'action:   reduces prolog and epilog code in procedures
'use:      to reduce unnecessary code and thus improve execution time.
'example:  leaf function(sys a, sys b) as sys '...
'remarks:  this is mostly applicable in 64 bit programming
'related:  external, export
'group:    directives



data       "#lookahead 13 43"
'action:   internally creates header declarations for all procedures.
'use:      allows procedure calls to forward reference
'example:  #lookahead
'          f "ok"
'          '...
'          sub f(s as string)
'            print s
'          end sub
'
'remarks:  procedures in inner blocks are ignored, so each block must
'          have its own '#lookahead'
'related:  scope
'group:    directives




data       "#autodim 13 44"
'action:   enable variables to be created without a Dim statement
'use:      switch off this mode for stricter compiling!
'example:  #autodim off
'remarks:  this mode is normally active.
'related:  
'group:    directives



data       "#alert 13 46"
'action:   generates a compiler message (like an error)
'use:      used to alert programmer about an area of code
'          that requires attention
'example:  #alert (please implement methods for this interface)
'related:  #error
'group:    directives



data       "#error 13 47"
'action:   generates a compiler error message
'use:      used to alert programmer about inconsistencies
'          that require attention.
'example:  #error (please implement methods for this interface)
'related:  #alert
'group:    directives



data       "#pragma 13 48"
'action:   ignored (C style compiler directives)
'use:      compilers use it for platform or compiler-specific mode changes
'example:  #pragma warning(push)
'related:  
'group:    directives



data       "#unique 13 49"
'action:   flags an error if a a symbol definition is not unique (in the same nesting level)
'use:      prevent symbols from being redefined
'example:  #unique enabled
'          #unique disabled 'default
'group:    directives



data       "#console 13 50"
'action:   programs compiled to EXE/DLL will run in an existing console (MS Subsystem 3)
'use:      prevents new console from being created. Inputs and outputs are with the existing console.
'example:  #console
'remarks:  When Oxygen is embedded it will use the Host's subsystem'
'group:    directives




'===========================
'>>O2H KEYWORDS
'===========================



data       "enum 14 1"
'action:   defines an enumeration structure (C syntax)
'use:      
'example:  see: defs
'related:  defs, typedef enum
'group:    Oxygen Basic structures


data       "let 14 2"
'action:   defines a variable or object
'use:      create a new variable or indirect object
'example:  let s="this is a string"
'          print typeof s 'result: 'string'
'related:  dim var
'remarks:  Similar to using 'dim' but the type is inferred from the assigned value.
'group:    Oxygen Basic structures



data       "operator 14 3"
'action:   define a new operator
'use:      create specialised operators for objects 
'example:  operator combine 5
'          '
'          'PRECEDENCE LEVELS:
'          '
'          'power        8 ^
'          'mult div     7 * / \
'          'add subtract 6 + /
'          'shifts       5 
'          'comparators  4 
'          'logicals     3 and 2 xor 1 or
'related:  method sub function
'remarks:  The number following the name set the precedence level (low to high):
'group:    Oxygen Basic keywords



data       "dim 14 4"
'action:   define a set of variables
'use:      
'example:  dim as string s="Hello World"
'related:  let var
'group:    Oxygen Basic keywords



data       "const 14 4"
'action:   define a set of constants
'use:      
'example:  const as string s="HelloWorld"
'related:  dim, #define
'group:    Oxygen Basic keywords



data       "local 14 05"
'action:   define a local set of variables
'use:      
'example:  
'related:  global, static
'group:    Oxygen Basic keywords



data       "static 14 06"
'action:   define a static set of variables, (persistant but invisible outsid the block)
'use:      
'example:  
'related:  global, local
'group:    Oxygen Basic keywords



data       "global 14 7"
'action:   define a set of variable that are visible throughout the program
'use:      
'example:  
'related:  static, local
'group:    Oxygen Basic keywords



data       "declare 14 8"
'action:   declare a procedure with its prototype (may be external or declared in advance)
'use:      
'example:  
'related:  library, lib, alias
'group:    Oxygen Basic keywords



data       "function 14 9"
'action:   define a function
'use:      
'example:  
'related:  sub, method
'group:    Oxygen Basic keywords: procedures



data       "sub 14 10"
'action:   define a sub. (like a function but not returning a value)
'use:      
'example:  
'related:  function, method
'group:    Oxygen Basic keywords: procedures



data       "method 14 11"
'action:   define a method. (a function or sub for objects)
'use:      
'example:  
'related:  function, sub
'group:    Oxygen Basic keywords: procedures



data       "end 14 12"
'action:   marks the end of a code block
'use:      terminating various block structures
'example:  if a>b then
'            b=a
'          end if
'related:  { .. }
'group:    Oxygen Basic keywords



data       "if 14 13"
'action:   start a conditional block with a test
'use:      conditional execution
'example:  if a<b then a=b
'related:  then, else, elseif, endif, while
'group:    Oxygen Basic: control structures



data       "then 14 14"
'action:   starts the conditional block where the prior test is met.
'use:      conditional execution
'example:  if a>b then print a
'related:  if, elseif, endif
'group:    Oxygen Basic: control structures



data       "else 14 15"
'action:   starts the alternative block where none of the prior conditions are met
'use:      conditional execution
'example:  if a>b then print a else print b 
'related:  if, elseif, endif
'group:    Oxygen Basic: control structures



data       "elseif 14 16"
'action:   make an alternative test if the previous condition was not met.
'use:      conditional execution
'example:  
'related:  if, else, endif
'group:    Oxygen Basic: control structures



data       "endif 14 17"
'same:     "end if"
'action:   end the conditional block
'use:      conditional execution
'example:  
'related:  if, then, elseif, else, end if
'group:    Oxygen Basic: control structures




data       "endsel 14 20" 'ALSO: "END SELECT"
'action:   end the select block
'use:      
'example:  
'related:  select, case
'group:    Oxygen Basic: control structures



data       "do 14 21"
'action:   start a block for repetition (looping)
'use:      
'example:  
'          a=0
'          do
'            '...
'            a+=1 : if a=4 then exit do
'          end do
'result:   a=4 '4 loops
'related:  while, continue do , exit do, enddo, end do
'group:    Oxygen Basic: control structures



data       "while 14 23"
'action:   start a block for conditional repetition
'use:      
'example:  
'          a=0
'          while a<4
'            a+=1
'          wend
'result:   a=4 '4 loops
'related:  do, continue do , exit do, wend, enddo, exit do
'group:    Oxygen Basic: control structures



data       "enddo 14 18"
data       "endwhile 14 19"
data       "wend 14 24" ' (NO `LOOP` KEYWORD)
'same:     "end do"
'same:     "end while"
'action:   end a block with conditional repetition (while..)
'use:      see: while
'related:  while, do, continue do , exit do
'group:    Oxygen Basic: control structures



data       "exit 14 25"
'action:   exit a Do or While block immediately
'use:      leave a loop after a condition has been met
'example:  
'          a=0 : b=1
'          do
'            a+=1
'            if a>4 then exit do
'            b+=b
'          end do
'result:   a=5 : b=16
'related:  do, while, continue do , exit do
'group:    Oxygen Basic: control structures



'itr another example showing low level usage of "exit"
'
'action:   exit a block of code
'use:      exit
'          exit 2
'example:  (
'            dec eax
'            jle exit
'            repeat
'          )
'result:   repeats block until eax<=0
'related:  repeat
'group:    control structures



data       "continue 14 26"
'action:   go back to the beginning of a Do or While block
'use:      "short circuit" a loop
'example:
'related:  do, while, exit do, wend, enddo
'group:    Oxygen Basic: control structures



data       "break  14 28"
'action:   exit a switch block
'related:  case, switch



data       "switch 14 29"
data       "select 14 29"
'action:   Start a Case block
'use:      
'example:  
'related:  case, endsel, end select
'group:    Oxygen Basic: control structures



data       "case 14 30"
'action:   specify a case to match followed by actions to perform
'use:      
'example:  
'related:  select, endsel, end select
'group:    Oxygen Basic: control structures



data       "for 14 31"
'action:   start an iteration block
'use:      
'example:  
'related:  to, step, next
'group:    Oxygen Basic: control structures



data       "to 14 32"
'action:   specify limit of an iteration
'use:      
'example:  
'related:  for, step, next
'group:    Oxygen Basic: control structures



data       "step 14 33"
'action:   specify increment of an iteration
'use:      for iterator = start to end step step increment
'example:  for i=1 to 10 step 2 : ... : next
'result:   i increeases by 2 with each cycle : 1 3 5 7 9
'comments: step can be omitted, a step of 1 is then assumed.
'          a negative step can also be used for down counting.
'related:  for, to, next
'group:    Oxygen Basic: control structures



data       "next 14 34"
'action:   end iteration block
'use:      
'example:  
'related:  for, to, step
'group:    Oxygen Basic: control structures



data       "goto 14 35"
'action:   jump to a specified label in the code
'use:      
'example:  
'related:  jmp, jmp fwd, gosub
'group:    Oxygen Basic: control structures



data       "gosub 14 36"
'action:   call a labelled subroutine
'use:      
'example:  
'related:  call
'group:    Oxygen Basic: procedures



data       "methods 14 37"
'action:   start a methods block for a class of objects.
'use:      
'example:  
'related:  method, class
'group:    procedures block
'group:    Oxygen Basic: structures



data       "return 14 38"
'action:   exit from a procedure returning a value (in a function or method)
'use:      
'example:  
'related:  ret
'group:    Oxygen Basic: procedures



data       "% 14 39" ' QB style equate
data       "$ 14 39" ' QB style equate
'action:   define an equate (which can be used as constants)
'use:      
'example:  
'related:  #define. #def. def
'group:    Oxygen Basic: macros



data       "block 14 41"
'action:   start a block
'use:      
'example:  
'related:  scope exit repeat do while if select
'group:    Oxygen Basic: code structures



data       "scope 14 42"
'action:   start a scope
'use:      create a block where variables and functions may be locally defined
'example:  :
'incfile1: basics\scope1.o2bas
'incfile2: basics\scope2.o2bas
'remarks:  when the scope ends any definitions created within the scope will be forgotten.
'related:  block exit repeat do while if select
'group:    Oxygen Basic: code structures



data       "#indexbase 14 44"
data       "indexbase 14 44"
'action:   define the index base for an array
'use:      most software assumes an indexbase of 0 or 1
'example:  indexbase 0
'remarks:  the default indexbase is 1 but C code normally assumes 0.
'          If the indexbase is defined inside a scope, such as a function then it
'          only applies within the scope and reverts to its previous value outside the scope.
'related:  dim #case #semicolon
'group:    Oxygen Basic: directives



data       "lib 14 45"
data       "library 14 45"
'action:   specify the name of a DLL library to associate with a set of procedure declarations
'use:      
'example:  lib "kernel32.dll"
'related:  declare
'group:    Oxygen Basic: directives



data       "extern 14 48"
'action:   associate declared procedures with a calling convention and/or dll name
'use:      
'example:  
'          extern stdcall lib "kernel32.dll"
'            declare function QueryPerformanceCounter(lpPerformanceCount as quad) as sys
'            declare function QueryPerformanceFrequency(lpPerformanceFrequency as quad) as sys
'          end extern
'remarks:  note ther is no need to use an Alias in these declarations if you give the exact name
'          of the procedures in their original case.
'related:  lib library calling
'group:    Oxygen Basic: linkage


data       "that 14 49"
'action:   suppress automatic "this" prefix inside OOP methods
'use:      resolving name conflict between an object member and another variable
'example:  this.voice=that voice
'          voice=that voice
'related:  method this
'group:    Objects


data       "this 14 50"
'action:   explicitly refer to an object's own members
'use:      distinguish a member from a local variable, param or function of the same name
'example:  this.len=len(s)
'incfile1: OOP\features\ThisThat.o2bas
'related:  method class that
'group:    Objects



data       "#noprec 14 53"
'action:   directive to evaluate an expression from left to right, ignoring standard operator precedence rules.
'use:      
'example:  
'related:  #noinit indexbase #byref #byval
'group:    Oxygen Basic: directives



data       "#noinit 14 54"
'action:   prevent variables from being automatically initialised to nul within a procedure
'use:      
'example:  
'related:  #noprec #byref #byval indexbase
'group:    Oxygen Basic: directives



data       "#byval 14 55"
'action:   pass parameter by value (not by its address)
'use:      
'example:  
'related:  #byval #noinit #noprec indexbase
'group:    Oxygen Basic: directives



data       "#byref 14 56"
'action:   pass parameter using its address instead of its value
'use:      
'example:  
'related:  #byref indexbase #noinit #noprec
'group:    Oxygen Basic: directives



data       "readstate 14 57"
'action:   read state records into compiler's state buffer.
'use:      normally used internally with dynamic compiling 
'example:
'remarks:  this instruction enables passes the locations
'          of parantal variables to a dynamically compiled program/function.
'related:  writestate compile
'group:    dynamic compiling



data       "writestate 14 58"
'action:   write state records from compiler's state buffer.
'use:      normally used internally with dynamic compiling 
'example:  
'remarks:  mapping information for the parantal variables of a function is embedded
'          as a string into the executable code. This can be read in during dynamic 
'          compiling.
'related:  readstate compile
'group:    dynamic compiling



data       "deff 14 59"
'action:   create an assembly code macro for the FPU  (metatype -17)
'use:      To create floating point maths functions
'example:  deff sine fsin
'remarks:  Deff macros make use of the FPU. All float functions are defined this way.
'          These are non-recursive macros and take no macro arguments. The expression parser is FPU- 
'          aware and takes care of passing parameters onto the FPU stack.
'related:  def macro sin cos
'group:    macros


'============
'METACOMMANDS
'============


data       "#if 13 100"
'action:   include block of code if conditions are met
'use:      
'example:  
'related:  #else, #elseif, #elif, #endif, #fi
'group:    meta control



data       "#else 13 101"
'action:   alternative block of code to include if prior conditions are not met.
'use:      
'example:  
'related:  #if, #elseif, #elif, #endif, #fi
'group:    meta control



data       "#elseif 13 102"
data       "#elif 13 102"
'action:   include block of code if these alternative conditions are met
'use:      
'example:  
'related:  #if, #else, #endif, #fi
'group:    meta control



data       "#endif 13 103"
data       "#fi 13 103"
'action:   end of conditional code inclusion block
'use:      
'example:  
'related:  #if, #elseif, #elif
'group:    meta control



data       "#define 13 104"
'action:   define a macro (C syntax)
'use:      
'example:  
'group:    macro



data       "#ifdef 13 105"
data       "#ifndef 13 106"
'action:   include code if symbol already defined
'use:      to allow blocks of code to be included or omitted at compile time
'example:  #ifdef MSWIN
'            #include "windows.inc"
'          #endif
'related:  #if, #elseif, #elif, #endif
'group:    meta control



data       "undef 13 107"
data       "#undef 13 107"
data       "#undefine 13 107"
'action:   remove a previously defined symbol
'use:      #undefine name
'example:  
'related:  #define, def, dim
'group:    macro



data       "alias 13 108" 'thinBasic headers
'action:   define another name for a name
'use:      alias original as newname
'example:  alias single as glFloat
'          dim as glFloat x,y,z
'related:  typedef
'group:    Oxygen Basic: linkage



data       "macro 13 109"
'action:   define a high level macro
'use:      many uses. Often a pseudo function for producing
'          inline code instead of a call
'example:  macro message(title,text) ( messagebox (0,text,title,0) )
'          message "Greeting","Hello!"
'related:  #define, #def, def
'group:    Oxygen Basic macro



data       "#blockdepth 13 201"
'action:   return block nesting depth.
'use:      to checking block depth at compile time.
'example:  #blockdepth node x 'use any descriptive label on the line
'remarks:  this command was introduced to catch unclosed blocks which
'          are often hard to trace.
'related:  scope block #recordof
'group:    compile-time diagnostics




data       "#recordof 13 202"
'action:   return internal record of a declared entity.
'use:      to check status at compile time.
'example:  #recordof MyStructure
'remarks:  this command was introduced to aid debugging at compile time
'          are often hard to trace.
'related:  #blockdepth
'group:    compile-time diagnostics





'================================
'primitives (Basic and C style
'================================


data "void      15 &h00" '*
data "byte      15 &h21"
data "ubyte     15 &h21"
data "string    15 &he1"
data "wstring   15 &he2"
data "bstring   15 &hc1"
data "gstr_     15 &hc1" '* Accept any Bstring width. for core functions only
data "bstr      15 &hc1"
data "bstring2  15 &hc2"
data "char      15 &ha1"
data "wchar     15 &ha2"
data "cstr_     15 &ha1" '* Accept any string width. for core functions only
data "asciiz    15 &ha1"
data "zstring   15 &ha1"
data "zstring2  15 &ha2"
data "asciiz    15 &ha1"
data "short     15 &h02"
data "long      15 &h04"
data "int       15 &h04"
data "integer   15 &h04"
data "float     15 &h64"
data "single    15 &h64"
data "double    15 &h68"
data "extended  15 &h6A"
data "ext       15 &h6A"
data "quad      15 &h48" '* using FPU (was 08)
data "word      15 &h22"
data "dword     15 &h24"
data "ulong     15 &h24"
data "uint      15 &h24"
data "qword     15 &h28"
data "any       15 &h08"
data "sys       15 &h08"
data "bool      15 &h04"
data "cpu       15 &h04" '*
data "fpu       15 &h40" '*
data "mmx       15 &h130" '* asm only
data "xmm       15 &h150" '* asm only
'
'group:    primitive types
'groupref: types



data       "signed 15 &h04"
'action:   specifies the type to be of signed integer
'use:      signed type variable
'example:  signed int v
'result:   a nsigned int (long) called v is created.
'group:    primitive types



data       "unsigned 15 &h24"
'action:   specifies the type to be of positive numbers only
'use:      unsigned type variable
'example:  unsigned long v
'result:   an unsigned long (dword) called v is created.
'group:    primitive types




'===========================================
'>>OVERLOADABLE INTEGER AND STRING FUNCTIONS
' overloadable functions -16
' integer and string
' TREATED AS STDCALL
'===========================================



data       "news 16 ##sys@a0 bstr 08"
'action:   allocates a bstring of null characters given the length in bytes
'use:      bstring=news length
'example:  b=news 1000
'          '...
'          frees b
'result:   b contains address of a string of 1000 nul characters . Then the sting is freed
'remarks:  Oxygen strings are automatically freed but Bstrings must be explicitly freed when no longer needed.
'related:  frees nuls space string
'group:    memory functions



data       "frees 16 ##bstring#@a0 sys 16 ##sys@a0 sys 16"
'action:   deallocates a bstring
'use:      frees bstring
'example:  b=news 1000
'          '...
'          frees b
'result:   b contains address of a string of 1000 nul characters . Then the sting is freed
'remarks:  Oxygen strings are automatically freed but Bstrings must be explicitly freed when no longer needed.
'related:  news
'group:    memory functions



data       "error 16 ##@ bstr 128"
'action:   returns a string containing the most recent compile or runtime errors (cleared at start of compiling)
'use:      to trap runtime errors
'example:  string e=error
'          if e then print e : 'followied by exit procedures
'result:   displays the last error, if it exists.
'remarks:  the error buffer is automatically cleared after using this function.
'related:  #error #alert
'group:    string functions



data       "nuls 16 ##sys@a0 bstr 08"
'action:   returns the address of a string of null characters given the length.
'use:      string=nuls length
'example:  s=nuls 1000
'result:   s is a string of 1000 null characters
'remarks:  for wide strings use string(n,wchr 0) instead
'related:  space string news frees
'group:    string functions



data       "compile 16 ##string@a0 sys 368"
'action:   compiles a string of source code and return address of executable machine code (which cn be called)
'use:      c=compile sourceString : call c
'example:  c=compile "basic : print `Hello` " : call c
'result:   when c is called `Hello` is displayed
'remarks:  If the compiling fails then error() will return a description of the problem.
'          The code has access to the parental local and static variables as well as globals.
'          When no longer used the code string should be deallocated by calling Frees c.
'
'          The dataspace reserved for data constants and string literals is limited to 16k bytes
'
'          This function is only available when Oxygen.dll is used as the runtime library. It is
'          not implemented in "independent" mode compilations.
'related:  readstate writestate
'group:    code functions



'data       "uncompile 16 ##string@a0 sys 376"
''action:   free a secondary compiled code string from memory and deduct its dataspace from the
''          data section (bss) of the host program.
''use:      remove the code when it is no longer required
''example:  uncompile TempProg
''remarks:  This should be used for code strings in the reverse order in which they were compiled.
''related:  compile readstate writestate
''group:    code functions



data       "val 16 ##zstring@a1 fpu 192"
'action:   returns numeric value of string
'use:      value=val string
'example:  v=val "2.5"
'result:   v=2.5
'relateds: str hex asc
'group:    string functions



data       "ltrim 16 ##string@a0 bstr 2312"
'action:   returns string with white space on the left trimmed off
'use:      string=rtrim(string)
'example:  s=ltrim "   abc"
'result:   s="abc"
'related:  rtrim space
'group:    string functions



data       "rtrim 16 ##string@a0 bstr 2320"
'action:   returns string with white space on the right trimmed off
'use:      string=rtrim(string)
'example:  s=rtrim "abc   "
'result:   s="abc"
'related:  ltrim space
'group:    string functions



data       "lcase 16 ##string@a0 bstr 2328"
'action:   returns lowercase of string
'use:      string=lcase(string)
'example:  s=lcase "ABCDEF"
'result:   s="abcdef"
'related:  ucase ltrim rtrim asc
'group:    string functions



data       "ucase 16 ##string@a0 bstr 2336"
'action:   returns uppercase of string
'use:      string=ucase(string)
'example:  s=ucase "abcdef"
'result:   s="ABCDEF"
'related:  lcase ltrim rtrim asc
'group:    string functions



data       "string 16 ##sys#sys@a0a0 bstr 2464 ##sys#zstring@a0a0 bstr 2472 ##sys#string@a0a0 bstr 2480"
'action:   returns a string of characters
'use:      string=string(length,character)
'example:  string=string(4,"a")
'result:   s="aaaa"
'related:  space nuls asc
'group:    string functions



data       "space 16 ##sys@a0 bstr 2488"
'action:   returns a string of spaces
'use:      string=space length
'example:  s=space 10
'result:   s="          " ' 10 spaces (ascii 32)
'remarks:  for wide strings use string(n,wchr 32) instead
'related:  nuls string ltrim rtrim
'group:    string functions



data       "left 16 ##gstr_#sys@a0a0 bstr 2344 ##cstr_#sys@a1a0 bstr 2400"
'action:   returns left part of a string by length
'use:      string=left string1,length
'example:  s=left "abcdef",3
'result:   s="abc"
'related:  mid ltrim rtrim
'group:    string functions



data       "mid 16 "+_
           "##gstr_#sys#sys@a0a0a0 bstr 2360 ##cstr_#sys#sys@a1a0a0 bstr 2392 "+_
           "##gstr_#sys@a0a0 bstr 2416 ##cstr_#sys@a1a0 bstr 2424 "
'name:     mid function
'action:   returns part of string1 at position with length
'use:      string=mid string1, position, length 'if length is omitted then the rest of string1 is returned
'example:  s=mid "abcdef",3 : t=mid "abcdef",3,2
'result:   s="cdef" : t="cd"
'remarks:  also works with untyped pointers
'related:   left ltrim rtrim instr _mid
'group:    string functions



''mid command

data       "_mid 16 ##gstr_#sys#gstr_@a0a0a0 sys 2456 ##cstr_#sys#gstr_@a1a0a0 sys 2352 "
'name:     mid command
'action:   patches string2 into string1 as position
'use:      mid(string1,position)=string2 : mid string1,position,string2
'example:  s="abcdef" : mid s,2,"BC"
'result:   s="aBCdef"
'related:  mid left ltrim rtrim
'group:    string command



data       "instr 16 ##string#string@a0a0 sys 2504 ##sys#string#string@a0a0a0 sys 320"
'action:   returns position of String2 within string1 searching from start position
'use:      location=instr(string1,string2) : location=instr(start,string1,string2)
'example:  p=instr("abcdef abcdef","def") : q=instr(8,"abcdef abcdef","def")
'result:   p=4 : q=11
'remarks:  also works with untyped pointers
'related:  replace mid
'group:    string functions



data       "getfile 16 ##cstr_@a1 bstr 336 ##cstr_#gstr_@a1a1 sys 2512"
'action:   copies file content to a string
'use:      String=getfile FileName : getfile FileName,String
'example:  string s : getfile "t.txt",s
'result:   s contains content of "t.txt"
'related:  putfile print
'group:    string functions



data       "putfile 16 ##cstr_#gstr_@a1a0 sys 344"
'action:   saves a string to a file
'use:      putfile FileName, String
'example:  putfile "t.txt","Hello"
'result:   A file named "t.xt" is created or overwritten containing "Hello"
'related:  getfile print
'group:    string functions




data       "print 16 ##cstr_@a1 sys 2496"
'action:   Displays strings and numbers
'use:      print String : print Number
'example:  print "ABC" : print 123 : print "ABC: " 123 " DEF: " 456
'result:   ABC
'          123
'          ABC: 123 DEF: 456
'remarks:  strings can be combined with numbers as sys as the first element is a string
'related:  val str hex putfile getfile
'group:    string functions



data       "asc 16 ##string@a0 sys 2432 ##string#sys@a0a0 sys 2368"
'action:   returns ascii encoding of a character in a string
'use:      AsciiCode=asc(String,CharacterPosition)
'example:  a=asc("ABCDEF",2)
'result:   a=66 'character='B'
'related:  unic chr val len str
'group:    string functions



data       "unic 16 ##wstring@a0 sys 2536 ##wstring#sys@a0a0 sys 2528"
'action:   returns encoding of a character in a wide string
'use:      uni=unic(String,CharacterPosition)
'example:  a=unic("ABCDEF",2)
'result:   a=0x0042 'character='B'
'related:  asc wchr chr val len str
'group:    string functions




data       "len 16 ##gstr_@a0 sys 2376 ##cstr_@a1 sys 2408" 'ITR MISSING LENGTH OF Z STRING?
'action:   returns length of string in charaters
'use:      length=len(string)
'example:  v=len("Hello")
'result:   v=5
'remarks:  characters may be one or two bytes (wide characters)
'related:  string space mid sizeof
'group:    string functions



data       "chr 16 ##sys@a0 bstr 2384"
'action:   returns string of 1 character of ascii encoding (0..255)
'use:      string=chr(AsciiValue)
'example:  s=chr(65)
'result:   s contains "A"
'related:  wchr asc unic string
'group:    string functions



data       "wchr 16 ##sys@a0 bstring2 2544"
'action:   returns wide string of a 2 byte character (encoding 0..65535 / 0xffff)
'use:      widestring=wchr(WideCharValue)
'example:  wstring ws=wchr(65)
'result:   ws contains unicode character 0x0041
'related:  unic asc string
'group:    string functions



data       "str 16 ##fpu@a0 bstr 2520 ##fpu#sys@a0a0 bstr 200 "
'action:   returns string representation of number
'use:      String=str(value)
'example:  s=str(-1.23456)   ' result: -1.23456
'          s=str(-1.23456,3) ' result: -1.235
'remarks:  rounding is automatically applied before decimal places are truncated.
'related:  hex val
'group:    string functions



data       "hex 16 ##fpu#sys@a0a0 bstr 216 ##fpu@a0 bstr 208 "
'action:   returns hexadecimal string representation of integer part of number
'use:      hexString=hex(value)
'example:  print hex(14.4)      'result 'E'
'          print hex(14.4, 4)   'result '000E'
'related:  str val
'group:    string functions



data       "guidval 16 ##guid#bstr@a1a0 sys 352  ##guid#zstring@a1a1 sys 352"
'action:   converts guid string to guidval
'use:      COM interfaces
'example:  guid clsid
'          guidval clsid,"{96749377-3391-11D2-9EE3-00C04F797396}"
'related:  guidtxt com
'group:    string functions



data       "guidtxt 16 ##guid@a1 bstr 360"
'action:   converts guidval to guidstring
'use:      COM interfaces
'example:  print guidtxt(clsid)
'result:   displays: {96749377-3391-11D2-9EE3-00C04F797396}
'related:  guidval com
'group:    string functions



data       "numberformat 16 ##sys#sys#sys#sys#sys#sys@a0a0a0a0@0@0 sys 2552 ##@ sys 2560 "
'action:   control how numbers are converted to strings
'use:      change the format of numbers
'example:  numberformat 3,0,0,0 : print 3.14159
'          'result: 3.142
'remarks:  there are 6 parameters:
'          1 decimal places (0..16)
'          2 strip trailing zeros (0 or 1)
'          3 always use scientic notation E format (0 or 1)
'          4 suppress 0 before decimal point: 0.1 becomes .1 (0 or 1)
'          5 insert extra leading space for non-negative numbers (0 or 1)
'          6 width allocated before decimal point: (0..31)
'            (inserting lead padding spaces)
'
'          when no parameters are given, it reverts to the default settings:
'          16,1,0,0,0,0
'          rounding is automatically performed before the decimal places are truncated.
'related:  print frac float
'group:    intrinsic procedure







'===========================
'>>FLOATING POINT MACRO
' macro function
'===========================


data       "abs 17 fabs"
'action:   returns the absolute value of a number (removes negative sign)
'use:      AbsoluteValue=abs(value)
'example:  a=abs(-2.5)
'result:   a=2.5
'related:  round frac trunc
'group:    floating point functions


data       "acos 17 fld st0 : fmul st0 : fld1 : fsubp st1,st0 : fchs : fsqrt : fxch : fpatan"
'action:   returns angle in radians given the ratio x/radius
'use:      angle=acos(YRRatio)
'example:  a=acos(0.5)
'result:   a=pi/3
'related:  cos asin atn atan
'group:    floating point functions



data       "asin 17 fld st0 : fmul st0 : fld1 : fsubp st1,st0 : fchs : fsqrt : fpatan"
'action:   returns angle in radians given the ratio y/radius
'use:      angle=asin(YRRatio)
'example:  a=asin(0.5)
'result:   a=pi/6
'related:  sin acos atn atan
'group:    floating point functions



data       "atan  17 fpatan"
'action:   returns angle in radians given the values of y and x
'use:      angle=atan(y,x)
'example:  a=atan(0.5,sqr(0.75))
'result:   a=pi/6
'related:  atn asin acos tan
'group:    floating point functions



data       "atn 17 fld1 : fpatan"
'action:   returns angle in radians given ratio y/x
'use:      angle=atn(YXRatio)
'example:  a=atn(1)*4
'result:   a=pi
'related:  atan asin acos tan
'group:    floating point functions



data       "cos 17 fcos"
'action:   returns cosine value (ratio of x/r)  given angle in radians
'use:      Cosine=cos(radians)
'example:  c=cos(pi/3)
'result:   c=0.5
'related:  sin tan
'group:    floating point functions



data       "deg 17 fldpi : fdivp st1 : push 180 : fimul dword [esp] : add esp,4"
'action:   returns degrees from value given in radians
'use:      degrees=deg(radians)
'example:  d=deg(pi)
'result:   180
'related:  rad atan
'group:    floating point functions



data       "frac 17 sub esp,8 : fstcw [esp] : fstcw [esp+2] : or [esp],0xc00 : fldcw [esp]"+cr+_
           "fld st0 : frndint : fsubp st1,st0 : fldcw [esp+2] : add esp,8"
'action:   returns the fractional part of a value
'use:      FractionValue=frac(value)
'example:  n=frac(123.456)
'result:   n=0.456
'related:  round abs
'group:    floating point functions



data       "hypot 17 fmul st0 : fxch : fmul st0 : faddp st1 : fsqrt"
'action:   returns the hypotenuse (longest side) of a right angle triangle given the other 2 sides.
'use:      hypotenuse=hypot(side1,side2)
'example:  h=hypot(3,4)
'result:   h=5
'related:  sqr pow
'group:    floating point functions



data       "lin 17 fldln2 : fxch : fyl2x"
data       "log 17 fldln2 : fxch : fyl2x"
'action:   returns the logarthm of the first value to base e (2.71828182845904523536.)
'use:      LogValue=log(Value)
'example:  a=log(10)
'result:   a=2.30258092...
'related:  lin2 log10 logn pow
'group:    floating point functions



data       "log2 17 fld1 : fxch : fyl2x"
'action:   returns the logarthm of the first value to base 2
'use:      LogValue=log2(Value)
'example:  a=log2(32)
'result:   a=5
'related:  log log10 logn pow
'group:    floating point functions


data       "log10 17 fldlg2 : fxch : fyl2x"
'action:   returns the logarthm of the first value to base 10
'use:      LogValue=log10(Value)
'example:  a=log10(100)
'result:   a=2
'group:    floating point functions



data       "logn 17 fld1 : fxch st1 : fyl2x : fld1 : fdivrp st1 : fxch st1 : fyl2x"
'action:   returns the logarthm of the first value to the base of the second value.
'use:      value=logn(value1,value2)
'example:  u=logn 1000,10) : v=logn(pow(10,1.5),10)
'result:   u=3 : v=1.5
'group:    floating point functions



data       "mod 17 fxch st1 : fprem : fstp st1"
'action:   returns the remainder of first value divided by the second value
'use:      modulus=mod(value1,value2)
'example:  m=mod(83.5, 10)
'result:   m=3.5
'related:  round trunc frac abs
'group:    floating point functions



data       "pi 17 fldpi"
'action:   returns pi, the ratio of the circumference of a circle to its diameter
'use:      p=pi ' or pi()
'example:  p=pi*2
'result:   p=6.283185...
'related:  tan atan
'group:    floating point functions



data       "exp 17 "+exponents
'action:   returns exponent of a value
'use:      a=exp(value)
'example:  a=exp(1)
'result:   a=2.718281828
'remarks:  This is the Euler number e, equivalent operation e^v
'related:  pow,log logn
'group:    floating point functions


data       "pow 17 "+powers
'action:   returns the value of the first value to the power of the second value
'use:      a=pow(value,exponent)
'example:  a=pow(2,3)
'result:   a=8
'remarks:  this is equivalent to 2^3
'related:  log logn
'group:    floating point functions



data       "rad 17 fldpi : fmulp st1 : push 180 : fidiv dword [esp] : add esp,4"
'action:   returns value in radians, given degrees
'use:      radians=rad(degrees)
'example:  v=rad(180)
'result:   v=pi
'related:  deg pi
'group:    floating point functions



data       "recip 17 fld1 : fdivrp"
'action:   returns reciprocal of value: a=1/v
'use:      reciprocal=recip(value)
'example:  a=recip(5/4)
'result:   a=0.8
'related:  frac round
'group:    floating point functions



data       "round 17 frndint"
'action:   returns a value rounded to the nearest whole number (rounded up or down)
'use:      WholeNumber=round(number)
'example:  m=round(1.49) : n=round(1.5)
'result:   m=1 : n=2
'related:  frac remain
'group:    floating point functions



data       "sin 17 fsin"
'action:   returns the sine of a value given in radians
'use:      Sine=sin(Radians)
'example:  v=sin(pi/6)
'result:   v=0.5
'related:  asin cos tan
'group:    floating point functions



data       "sqr 17 fsqrt"
data       "sqrt 17 fsqrt"
'action:   returns the square root of a value
'use:      SquareRoot=sqr(value)
'example:  v=sqr(9)
'result:   3
'remarks:  sqr and sqrt are the same
'related:  pow log hypot
'group:    floating point functions



data       "tan 17 fptan : fstp st0"
'action:   returns the tangent of a value given in radians
'use:      tangent=tan(radians)
'example:  t=tan(a)
'result:   t= ratio y/x
'related:  atan atn sin cos
'group:    floating point functions


data       "trunc 17 sub esp,8 : fstcw [esp] : fstcw [esp+2] : or [esp],0xc00 : fldcw [esp]"+cr+_
           "frndint : fldcw [esp+2] : add esp,8"
'action:   truncate the fractional part of a value and return it
'use:      IntegerValue=trunc(FloatValue)
'example:  i=trunc(123.456)
'result:   i=123
'related:  round trunc
'group:    floating point functions






'==================
'SECONDARY KEYWORDS
'==================

'the data is for information only

#if 0


data       "inherits 54"
data       "of 54"
data       "from 54"
'related:  of from has class
'group:    secondary keyword
'groupref  SingleInherit
'remarks:  'of', 'has' and 'from' currently means 'inherits. They 
'          indicate derivation from a single parental class.
'          COM interfaces us single inheritance



data       "has 54"
'related:  of from inherits class
'group:    secondary keyword
'remarks:  'has' is used when the class is derived from more than a single
'          parental parental class. ie: multiple inheritance.



data       "lib 54"
'related:  library declare extern alias link at
'group:    secondary keyword
'remarks:  lib may be specified in the Declare statement or it may be
'          specified in an extern statement, referring to an entire group
'          of declarations.



data       "alias 54"
'related:  library declare external alias
'group:    secondary keyword
'remarks:  if the procedure name is an exact case match then it is not
'          necessary to specify an Alias.



data       "export 54"
'related:  external function sub
'group:    secondary keyword
'remarks:  procedures with this attribute are visible to external callers
'          when they are compiled as part of a DLL (Dynamic Link Library)



data:      "at 54"
'related:  link external export lib
'group:    secondary keyword
'remarks:  used to specify the loction of the procedure.
'          This could be a register operand [ebx+4] or a simple variable


data:      "link 54"
'related:  at external export lib
'group:    secondary keyword
'remarks:  used to the address of the procedure to a simple variable


data       "external 54"
'related:  export callback com virtual function
'group:    secondary keyword
'remarks:  functions with the external attribute expect to be called from
'          procedures external to Oxygen. Additional register management is
'          enlisted to support external calls.
'
'          Objects of classes with this attribute have virtual function table
'          pointers embedded in their structures to enable their methods to
'          be invoked externally. For internally used objects, the methods table
'          is known to the objects at compile time so VFT pointers are not used.


data       "com 54"
'related:  virtual external export class of inherits
'group:    secondary keyword
'remarks:  COM is a protocol used by Microsoft to provide Object interfaces. Classes
'          with this attribute are considered Virtual.



data       "virtual 54"
'related:  com external export class
'group:    secondary keyword
'remarks:  objects of virtual classes are usually created by some
'          kind of server or class factory. The client only receives
'          a reference (pointer) to the object from the server and only knows
'          how to invoke its methods. It assumes no knowledge of its inner workings.



data       "public 54"
'use:      determines the scope of a class member
'
'related:  private protected com external export class
'group:    secondary keyword
'remarks:  public members (default) are visible to ouside the object.



data       "protected 54"
'use:      determines the scope of a class member
'related:  private public com external export class
'group:    secondary keyword
'remarks:  protected members are only visible inside objects of the class or derived class.



data       "private 54"
'use:      determines the scope of a class member
'related:  protected public com external export class
'group:    secondary keyword
'remarks:  private members are visible only inside objects of the class.



data       "dim 53"
'groupref: dimsyntax
'group:    creating variables
'incfile1: basics\DimSyntax.o2bas
'remarks:  C style instantiations is a simpler alternative to using 'Dim'.
'
'          When autodim is active (by default), variables can be automatically 
'          instantiated unless they are arrays or referenced (byref) variables.



data       "type 53"
'use:      specify compund structure for a variable
'groupref: inhstructures
'group:    inherited structures
'incfile1: basics\types2.o2bas



data       "once 54"
'action:   ensures that a file is included in the source code one only.
'example:  #include once "../../MinWin.inc"
'related:  #include includepath
'groupref: includes
'group:    secondary keyword



data       "$ 53"
data       "% 53"
'groupref: equates
'group:    equates
'incfile1: basics\equates.o2bas



data       "arrays 99"
'groupref: arrays
'group:    creating arrays
'incfile1: basics\Arrays.o2bas



data       "if 52"
data       "then"
data       "elseif"
data       "else"
data       "endif" 'itr end if
'groupref: conditionals
'group:    conditionals
'incfile1: basics\Conditionals.o2bas



data       "do 52"
data       "while 52"
data       "exit 52"
data       "continue 52"
data       "wend 52"
data       "enddo 52" 'itr enddo
'groupref: loops
'group:    loops
'incfile1: basics\loops.o2bas



data       "for 52"
data       "to  52"
data       "step 52"
data       "next 52"
'groupref: iteration
'group:    iteration
'incfile1: basics\iteration.o2bas



data       "select 52"
data       "case 52"
data       "case else 52" 'itr case else
data       "endselect 52" 'itr end select
'groupref: selection
'group:    selection
'incfile1: basics\select.o2bas



data       "type 53"
'groupref: structures
'group:    structures
'incfile1: basics\types1.o2bas


'---------
'OPERATORS
'=========
'
'
data "load 51"
data "= 51"
data ":= 51"
data "+ 51"
data "- 51"
data "* 51"
data "/ 51"
data "\ 51"
data "+= 51"
data "-= 51"
data "*= 51"
data "/= 51"
data "== 51"
data "!= 51" 'itr check
data "<> 51"
data "< 51"
data "> 51"
data "<= 51"
data "> 51"
data ">= 51"
data "<= 51"
data "and 51"
data "or 51"
data "xor 51"
data "and= 51"
data "or= 51"
data "xor=" 51"
data "& 51"
data "| 51"
data "&= 51"
data "|= 51"
data "&& 51"
data "|| 51"
data "<< 51"
data ">> 51"
'data "^^ 51" 'itr logical xor ??
'
'groupref: operators
'group:    operators



'-------------------
'CALLING CONVENTIONS
'===================


data "stdcall 55"
data "cdecl 55"
data "ms64 55"
data "pascal 55"
data "fpu 55"
'groupref: calling
'group:    calling conventions



#endif


'===========================

data "" ,0,0 'mark end of data

'===========================
'>>REGISTERS
'===========================

x86_registers:

' regsize regno regtype

data "al    10 101"
data "cl    10 111"
data "dl    10 121"
data "bl    10 131"
data "ah    10 141"
data "ch    10 151"
data "dh    10 161"
data "bh    10 171"
'---------------------------
data "ax    10 201"
data "cx    10 211"
data "dx    10 221"
data "bx    10 231"
data "sp    10 241"
data "bp    10 251"
data "si    10 261"
data "di    10 271"
'---------------------------
data "eax   10 401"
data "ecx   10 411"
data "edx   10 421"
data "ebx   10 431"
data "esp   10 441"
data "ebp   10 451"
data "esi   10 461"
data "edi   10 471"
'---------------------------
data "st0   10 A02"
data "st1   10 A12"
data "st2   10 A22"
data "st3   10 A32"
data "st4   10 A42"
data "st5   10 A52"
data "st6   10 A62"
data "st7   10 A72"
'---------------------------
data "mmx0  10 F03" 
data "mmx1  10 F13" 
data "mmx2  10 F23" 
data "mmx3  10 F33" 
data "mmx4  10 F43" 
data "mmx5  10 F53" 
data "mmx6  10 F63" 
data "mmx7  10 F73" 
'---------------------------  
data "xmm0  10 F04"
data "xmm1  10 F14"
data "xmm2  10 F24"
data "xmm3  10 F34"
data "xmm4  10 F44"
data "xmm5  10 F54"
data "xmm6  10 F64"
data "xmm7  10 F74"
data "xmm8  10 F84"
data "xmm9  10 F94"
data "xmm10 10 FA4"
data "xmm11 10 FB4"
data "xmm12 10 FC4"
data "xmm13 10 FD4"
data "xmm14 10 FE4"
data "xmm15 10 FF4"
'---------------------------
data "cr0   10 405" '129  
data "cr2   10 415" 
data "cr3   10 425" 
data "cr4   10 435" 
'---------------------------
data "dr0   10 406" '130  
data "dr1   10 416" 
data "dr2   10 426" 
data "dr3   10 436" 
data "dr4   10 446"
data "dr5   10 456" 
data "dr6   10 466" 
data "dr7   10 476" 
'---------------------------
data "es    10 407"'131  
data "cs    10 417" 
data "ss    10 427" 
data "ds    10 437" 
data "fs    10 447"
data "gs    10 457" 
'---------------------------
'64 bit mode registers
'---------------------------
data "rax   10 809"
data "rcx   10 819"
data "rdx   10 829"
data "rbx   10 839"
data "rsp   10 849"
data "rbp   10 859"
data "rsi   10 869"
data "rdi   10 879"
data "r8    10 889"
data "r9    10 899"
data "r10   10 8A9"
data "r11   10 8B9"
data "r12   10 8C9"
data "r13   10 8D9"
data "r14   10 8E9"
data "r15   10 8F9"
'---------------------------
data "r8l   10 489"
data "r9l   10 499"
data "r10l  10 4A9"
data "r11l  10 4B9"
data "r12l  10 4C9"
data "r13l  10 4D9"
data "r14l  10 4E9"
data "r15l  10 4F9"
'---------------------------
data "r8w   10 289"
data "r9w   10 299"
data "r10w  10 2A9"
data "r11w  10 2B9"
data "r12w  10 2C9"
data "r13w  10 2D9"
data "r14w  10 2E9"
data "r15w  10 2F9"
'---------------------------
data "r8d   10 489"
data "r9d   10 499"
data "r10d  10 4A9"
data "r11d  10 4B9"
data "r12d  10 4C9"
data "r13d  10 4D9"
data "r14d  10 4E9"
data "r15d  10 4F9"
'
'groupref: x86Registers
'action:   access CPU registers directly using Assembly code
'use:      
'example:  mov eax,42
'          add eax,ecx
'result:   eax now contains 42 plus the value in ecx
'remarks:  many of these registers are only accessible if the CPU
'          is runnig in a particular mode - eg: the R* registers require
'          64 bit mode.
'group:    X86 CPU registers



'=================================
data "" 'end of data x86 registers
'=================================



'===
'END
'===


